# -*- coding: utf-8 -*-

# This file is part of convertdate.
# http://github.com/fitnr/convertdate

# Licensed under the MIT license:
# http://opensource.org/licenses/MIT
# Copyright (c) 2016, fitnr <fitnr@fakeisthenewreal>

# source: Wikipedia
# month 13 is the Sansculottides
french_republican_days = {
    1: [
        "Raisin",
        "Safran",
        "Châtaigne",
        "Colchique",
        "Cheval",
        "Balsamine",
        "Carotte",
        "Amaranthe",
        "Panais",
        "Cuve",
        "Pomme de terre",
        "Immortelle",
        "Potiron",
        "Réséda",
        "Âne",
        "Belle de nuit",
        "Citrouille",
        "Sarrasin",
        "Tournesol",
        "Pressoir",
        "Chanvre",
        "Pêche",
        "Navet",
        "Amaryllis",
        "Bœuf",
        "Aubergine",
        "Piment",
        "Tomate",
        "Orge",
        "Tonneau",
    ],
    2: [
        "Pomme",
        "Céleri",
        "Poire",
        "Betterave",
        "Oie",
        "Héliotrope",
        "Figue",
        "Scorsonère",
        "Alisier",
        "Charrue",
        "Salsifis",
        "Mâcre",
        "Topinambour",
        "Endive",
        "Dindon",
        "Chervis",
        "Cresson",
        "Dentelaire",
        "Grenade",
        "Herse",
        "Bacchante",
        "Azerole",
        "Garance",
        "Orange",
        "Faisan",
        "Pistache",
        "Macjonc",
        "Coing",
        "Cormier",
        "Rouleau",
    ],
    3: [
        "Raiponce",
        "Turneps",
        "Chicorée",
        "Nèfle",
        "Cochon",
        "Mâche",
        "Chou-fleur",
        "Miel",
        "Genièvre",
        "Pioche",
        "Cire",
        "Raifort",
        "Cèdre",
        "Sapin",
        "Chevreuil",
        "Ajonc",
        "Cyprès",
        "Lierre",
        "Sabine",
        "Hoyau",
        "Érable à sucre",
        "Bruyère",
        "Roseau",
        "Oseille",
        "Grillon",
        "Pignon",
        "Liège",
        "Truffe",
        "Olive",
        "Pelle",
    ],
    4: [
        "Tourbe",
        "Houille",
        "Bitume",
        "Soufre",
        "Chien",
        "Lave",
        "Terre végétale",
        "Fumier",
        "Salpêtre",
        "Fléau",
        "Granit",
        "Argile",
        "Ardoise",
        "Grès",
        "Lapin",
        "Silex",
        "Marne",
        "Pierre à chaux",
        "Marbre",
        "Van",
        "Pierre à plâtre",
        "Sel",
        "Fer",
        "Cuivre",
        "Chat",
        "Étain",
        "Plomb",
        "Zinc",
        "Mercure",
        "Crible",
    ],
    5: [
        "Lauréole",
        "Mousse",
        "Fragon",
        "Perce-neige",
        "Taureau",
        "Laurier-thym",
        "Amadouvier",
        "Mézéréon",
        "Peuplier",
        "Coignée",
        "Ellébore",
        "Brocoli",
        "Laurier",
        "Avelinier",
        "Vache",
        "Buis",
        "Lichen",
        "If",
        "Pulmonaire",
        "Serpette",
        "Thlaspi",
        "Thimelé",
        "Chiendent",
        "Trainasse",
        "Lièvre",
        "Guède",
        "Noisetier",
        "Cyclamen",
        "Chélidoine",
        "Traîneau",
    ],
    6: [
        "Tussilage",
        "Cornouiller",
        "Violier",
        "Troène",
        "Bouc",
        "Asaret",
        "Alaterne",
        "Violette",
        "Marceau",
        "Bêche",
        "Narcisse",
        "Orme",
        "Fumeterre",
        "Vélar",
        "Chèvre",
        "Épinard",
        "Doronic",
        "Mouron",
        "Cerfeuil",
        "Cordeau",
        "Mandragore",
        "Persil",
        "Cochléaria",
        "Pâquerette",
        "Thon",
        "Pissenlit",
        "Sylvie",
        "Capillaire",
        "Frêne",
        "Plantoir",
    ],
    7: [
        "Primevère",
        "Platane",
        "Asperge",
        "Tulipe",
        "Poule",
        "Bette",
        "Bouleau",
        "Jonquille",
        "Aulne",
        "Couvoir",
        "Pervenche",
        "Charme",
        "Morille",
        "Hêtre",
        "Abeille",
        "Laitue",
        "Mélèze",
        "Ciguë",
        "Radis",
        "Ruche",
        "Gainier",
        "Romaine",
        "Marronnier",
        "Roquette",
        "Pigeon",
        "Lilas",
        "Anémone",
        "Pensée",
        "Myrtille",
        "Greffoir",
    ],
    8: [
        "Rose",
        "Chêne",
        "Fougère",
        "Aubépine",
        "Rossignol",
        "Ancolie",
        "Muguet",
        "Champignon",
        "Hyacinthe",
        "Râteau",
        "Rhubarbe",
        "Sainfoin",
        "Bâton d'or",
        "Chamerisier",
        "Ver à soie",
        "Consoude",
        "Pimprenelle",
        "Corbeille d'or",
        "Arroche",
        "Sarcloir",
        "Statice",
        "Fritillaire",
        "Bourrache",
        "Valériane",
        "Carpe",
        "Fusain",
        "Civette",
        "Buglosse",
        "Sénevé",
        "Houlette",
    ],
    9: [
        "Luzerne",
        "Hémérocalle",
        "Trèfle",
        "Angélique",
        "Canard",
        "Mélisse",
        "Fromental",
        "Martagon",
        "Serpolet",
        "Faux",
        "Fraise",
        "Bétoine",
        "Pois",
        "Acacia",
        "Caille",
        "Œillet",
        "Sureau",
        "Pavot",
        "Tilleul",
        "Fourche",
        "Barbeau",
        "Camomille",
        "Chèvrefeuille",
        "Caille-lait",
        "Tanche",
        "Jasmin",
        "Verveine",
        "Thym",
        "Pivoine",
        "Chariot",
    ],
    10: [
        "Seigle",
        "Avoine",
        "Oignon",
        "Véronique",
        "Mulet",
        "Romarin",
        "Concombre",
        "Échalote",
        "Absinthe",
        "Faucille",
        "Coriandre",
        "Artichaut",
        "Girofle",
        "Lavande",
        "Chamois",
        "Tabac",
        "Groseille",
        "Gesse",
        "Cerise",
        "Parc",
        "Menthe",
        "Cumin",
        "Haricot",
        "Orcanète",
        "Pintade",
        "Sauge",
        "Ail",
        "Vesce",
        "Blé",
        "Chalémie",
    ],
    11: [
        "Épeautre",
        "Bouillon blanc",
        "Melon",
        "Ivraie",
        "Bélier",
        "Prêle",
        "Armoise",
        "Carthame",
        "Mûre",
        "Arrosoir",
        "Panic",
        "Salicorne",
        "Abricot",
        "Basilic",
        "Brebis",
        "Guimauve",
        "Lin",
        "Amande",
        "Gentiane",
        "Écluse",
        "Carline",
        "Câprier",
        "Lentille",
        "Aunée",
        "Loutre",
        "Myrte",
        "Colza",
        "Lupin",
        "Coton",
        "Moulin",
    ],
    12: [
        "Prune",
        "Millet",
        "Lycoperdon",
        "Escourgeon",
        "Saumon",
        "Tubéreuse",
        "Sucrion",
        "Apocyn",
        "Réglisse",
        "Échelle",
        "Pastèque",
        "Fenouil",
        "Épine vinette",
        "Noix",
        "Truite",
        "Citron",
        "Cardère",
        "Nerprun",
        "Tagette",
        "Hotte",
        "Églantier",
        "Noisette",
        "Houblon",
        "Sorgho",
        "Écrevisse",
        "Bigarade",
        "Verge d'or",
        "Maïs",
        "Marron",
        "Panier",
    ],
    13: [
        "La Fête de la Vertu",
        "La Fête du Génie",
        "La Fête du Travail",
        "La Fête de l'Opinion",
        "La Fête des Récompenses",
        "La Fête de la Révolution",
    ],
}
